/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.List;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.isobus.model.IsobusPartField;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.Lookups;

/**
 * Creates a node for each partfield under the Partfields aggregate node.
 * 
 * Each partFieldNode should be draggable.
 * 
 * @author Gerben Feenstra
 */
@Messages({
"isobus.project.taskdata.node.name.partfield_node={0} - {1}"
})

public class PartFieldNodeFactory extends ChildFactory<Node> {

    private final List<IsobusPartField> partFields;
    private static final String PARTFIELD_CHILDNODE_ICON = "nl/cloudfarming/client/isobus/type/isobus_taskdata_partfield.png";
    private static final String PARTFIELD_LAYER_NAME = "IsobusPartfield_layer";
        
    public PartFieldNodeFactory(List<IsobusPartField> partFields) {
        this.partFields = partFields;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        for(IsobusPartField isobusPartField : partFields){
            String displayName = NbBundle.getMessage(PartFieldNodeFactory.class, "isobus.project.taskdata.node.name.partfield_node", isobusPartField.getCode(), isobusPartField.getDesignator());
            toPopulate.add(new PartFieldNode(isobusPartField, displayName));
        }        
        return true;
    }

    @Override
    protected Node createNodeForKey(Node node) {
        return node;
    }
    
    /**
     * A node for an Isobus PartField
     */
    private static final class PartFieldNode extends AbstractNode {
        /**
         * Create a PartFieldNode and specify a layer for this node in the lookup
         * @param isobusPartField the isoBusPartField this node has to represent.
         * @param displayName Name to display for this node
         */
        public PartFieldNode(IsobusPartField isobusPartField, String displayName) {
            super( Children.create(new PartFieldChildFactory(isobusPartField), true), Lookups.fixed(new SimpleLayer(PARTFIELD_LAYER_NAME), new PartFieldGeographical(isobusPartField)));
            this.setDisplayName(displayName);
            this.setIconBaseWithExtension(PARTFIELD_CHILDNODE_ICON);
        }

        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        @Override
        public Transferable clipboardCopy() throws IOException {
            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        }   
        
        // TODO: Specialized help page for isobus field-nodes?
        @Override
        public HelpCtx getHelpCtx() {
            return new HelpCtx("nl.cloudfarming.client.isobus.project.explorer");
        }
    }
}
